<?php

/**
* NOTA SOBRE LA LICENCIA DE USO DEL SOFTWARE
* 
* El uso de este software está sujeto a las Condiciones de uso de software que
* se incluyen en el paquete en el documento "Aviso Legal.pdf". También puede
* obtener una copia en la siguiente url:
* http://www.redsys.es/wps/portal/redsys/publica/areadeserviciosweb/descargaDeDocumentacionYEjecutables
* 
* Redsys es titular de todos los derechos de propiedad intelectual e industrial
* del software.
* 
* Quedan expresamente prohibidas la reproducción, la distribución y la
* comunicación pública, incluida su modalidad de puesta a disposición con fines
* distintos a los descritos en las Condiciones de uso.
* 
* Redsys se reserva la posibilidad de ejercer las acciones legales que le
* correspondan para hacer valer sus derechos frente a cualquier infracción de
* los derechos de propiedad intelectual y/o industrial.
* 
* Redsys Servicios de Procesamiento, S.L., CIF B85955367
*/

include_once ABSPATH.'/wp-content/plugins/redsys/wc-redsys-ref.php';
include_once ABSPATH.'/wp-content/plugins/redsys/ApiRedsysIS/initRedsysApi.php';

class WC_Redsys extends WC_Payment_Gateway {

    public function __construct() {
        $this->id                 = 'redsys';
        //$this->icon               = home_url() . '/wp-content/plugins/redsys/pages/assets/images/Redsys.png';
        $this->method_title       = __( 'Pago con Tarjeta (REDSYS)', 'woocommerce' );
        $this->method_description = __( 'Esta es la opción de la pasarela de pago de Redsys.', 'woocommerce' );
        $this->process_url        = add_query_arg( 'wc-api', 'WC_redsys_process', home_url( '/' ) );
        $this->ref_process_url        = add_query_arg( 'wc-api', 'WC_redsys_ref_process', home_url( '/' ) );
        $this->secure_redir_url        = add_query_arg( 'wc-api', 'WC_redsys_secure_redir', home_url( '/' ) );
        $this->secure_back_url        = add_query_arg( 'wc-api', 'WC_redsys_secure_back', home_url( '/' ) );

        $this->has_fields         = false;

        $this->version			  =  WC_Redsys::getVersion();
        
        // Load the settings
        $this->init_settings();
        $this->init_form_fields();

        $this->title              = $this->get_option( 'title' );
        $this->description        = $this->get_option( 'description' );

        // Get settings
        $this->entorno            = $this->get_option( 'entorno' );
        $this->nombre             = $this->get_option( 'name' );
        $this->fuc                = $this->get_option( 'fuc' );
        $this->tipopago           = $this->get_option( 'tipopago' );
        $this->clave256           = $this->get_option( 'clave256' );
        $this->terminal           = $this->get_option( 'terminal' );
        $this->moneda             = $this->get_option( 'moneda' );
        $this->trans              = $this->get_option( 'trans' );
        $this->activar_log	  	  = $this->get_option( 'activar_log' );
        $this->idioma             = $this->get_option( 'idioma' );
        $this->estado             = $this->get_option( 'estado' );
//         $this->withdcc            = $this->get_option( 'withdcc' );
        $this->with3ds            = $this->get_option( 'with3ds' );
        $this->withref            = $this->get_option( 'withref' );

        $this->button_text        = $this->get_option( 'button_text' );
        $this->button_style       = $this->get_option( 'button_style' );
        $this->body_style         = $this->get_option( 'body_style' );
        $this->form_style          = $this->get_option( 'form_style' );
        $this->form_text_style    = $this->get_option( 'form_text_style' );


        // Actions
        add_action( 'woocommerce_receipt_redsys', array( $this, 'receipt_page' ) );
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
        //Payment listener/API hook
        add_action( 'woocommerce_api_wc_redsys_process', array( $this, 'process_usual_order' ) );
        add_action( 'woocommerce_api_wc_redsys_ref_process', array( $this, 'process_reference_order' ) );
        add_action( 'woocommerce_api_wc_redsys_secure_redir', array( $this, 'redirect_to_tdsecure' ) );
        add_action( 'woocommerce_api_wc_redsys_secure_back', array( $this, 'back_from_tdsecure' ) );
    }
    
    static function getVersion(){
    	$content=file_get_contents(ABSPATH.'/wp-content/plugins/redsys/class-wc-redsys.php');
    	$matches=preg_grep("/\* Version: /", explode("\n", $content));
    	foreach($matches as $match){
    		return substr($match, -5, 5);
    	}
    	
    	return "";
    }
    
    function process_usual_order(){
    	ISLogger::generateIdLog(true);
    	$origIdCart=$_POST["origIdCart"];
    	$order=new WC_Order($origIdCart);

        //Calculo del precio total del pedido
        $transaction_amount = number_format( (float) ($order->get_total()), 2, '.', '' );
        $transaction_amount = str_replace('.','',$transaction_amount);
        $transaction_amount = floatval($transaction_amount);
        
        $productos="";
        $products = WC()->cart->cart_contents;
        foreach ($products as $product) {
            $productos .= $product['quantity'].'x'.$product['data']->post->post_title.'/';
        }
    	
    	$request = new ISOperationMessage ();
    	$request->setAmount ( $transaction_amount );
    	$request->setCurrency ( $this->moneda );
    	$request->setMerchant ( $this->fuc  );
    	$request->setTerminal ( $this->terminal  );
    	$request->setOrder ( $_POST["idCart"] );
    	$request->setOperID ( $_POST["operID"] );
    	$request->setTransactionType ( $this->trans );
		$request->addParameter ( "DS_MERCHANT_TITULAR", $order -> billing_first_name." ".$order -> billing_last_name );
		$request->addParameter ( "DS_MERCHANT_PRODUCTDESCRIPTION", $productos );
		$request->addParameter ( "DS_MERCHANT_MODULE", "WO_redsys_" . $this->version );
		$ip = $_SERVER['REMOTE_ADDR'] == "::1" ? "127.0.0.1" : $_SERVER['REMOTE_ADDR'];
		$request->addParameter ( "DS_MERCHANT_CLIENTIP", $ip );
		
		if($this->withref && $_POST["save"] === "true") {
			$request->createReference ();
			ISLogger::debug("Marcamos que queremos generar referencia. Recibimos por POST: " . $_POST["save"]);
		}
		
		if($this->with3ds){
// 			$request->useSecurePayment ();

			//Pendiente de que se reciba este parámetro que llegará junto al idOper
			$json3DS='{
				"threeDSInfo":"AuthenticationData",
				"protocolVersion":"1.0.2",
				"browserAcceptHeader":"text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8,application/json",
				"browserUserAgent":"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36"
			}';

			$request->addEmvParameters(json_decode($json3DS,true));
		}

		else 
			$request->useDirectPayment ();
		
// 		if($this->withdcc)
// 			$request->withDcc();
		
		$service = new ISService ( $this->clave256, $this->entorno );
		$result = $service->sendOperation ( $request );
		
		if ($result->getResult () == ISConstants::$RESP_LITERAL_OK) {
			$order->update_status($this->estado,__( 'Order accepted: payment registered successfully', 'woocommerce' ));
			
			$reference=$result->getOperation()->getMerchantIdentifier();
			if($reference!=null){
				$idCustomer=get_post_meta($origIdCart, '_customer_user', true);
				$cardNumber=$result->getOperation()->getCardNumber();
				$brand=$result->getOperation()->getCardBrand();
				$cardType=$result->getOperation()->getCardType();
				
				WC_Redsys_Ref::saveReference($idCustomer, $reference, $cardNumber, $brand, $cardType);
			}
						
	    	header('Content-Type: application/json');
			die(json_encode(array("redir"=>true, "url"=>$this->get_return_url($order))));
		}
		else{
			if ($result->getResult () == ISConstants::$RESP_LITERAL_AUT) {
				WC()->session->set("REDSYS_oper", serialize ( $result->getOperation () ));
				WC()->session->set('REDSYS_pareq', $result->getOperation ()->getPaRequest ());
				WC()->session->set('REDSYS_urlacs', $result->getOperation ()->getAcsUrl ());
				WC()->session->set('REDSYS_md', $result->getOperation ()->getAutSession ());
				WC()->session->set('REDSYS_idCart', $origIdCart);
				
		    	header('Content-Type: application/json');
				die(json_encode(array("redir"=>false, "url"=>$this->secure_redir_url)));
			} else {
				$order->update_status('cancelled',__( 'Order cancelled: denied operation', 'woocommerce' ));
				
		    	header('Content-Type: application/json');
				die(json_encode(array("redir"=>true, "url"=>str_replace("&amp;","&",$order->get_cancel_order_url()))));
			}
		}
    }

    function process_reference_order(){
    	ISLogger::generateIdLog(true);
    	$origIdCart=$_POST["origIdCart"];
    	$order=new WC_Order($origIdCart);
    	
		$userId=get_post_meta($origIdCart, '_customer_user', true);
		$ref=WC_Redsys_Ref::getCustomerRef($userId);
    
    	//Calculo del precio total del pedido
    	$transaction_amount = number_format( (float) ($order->get_total()), 2, '.', '' );
    	$transaction_amount = str_replace('.','',$transaction_amount);
    	$transaction_amount = floatval($transaction_amount);
    
    	$productos="";
    	$products = WC()->cart->cart_contents;
    	foreach ($products as $product) {
    		$productos .= $product['quantity'].'x'.$product['data']->post->post_title.'/';
    	}
    	 
    	$request = new ISOperationMessage ();
    	$request->setAmount ( $transaction_amount );
    	$request->setCurrency ( $this->moneda );
    	$request->setMerchant ( $this->fuc  );
    	$request->setTerminal ( $this->terminal  );
    	$request->setOrder ( $_POST["idCart"] );
    	$request->setTransactionType ( $this->trans );
    	$request->useDirectPayment ();
    	$request->addParameter ( "DS_MERCHANT_TITULAR", $order -> billing_first_name." ".$order -> billing_last_name );
    	$request->addParameter ( "DS_MERCHANT_PRODUCTDESCRIPTION", $productos );
    	$request->addParameter ( "DS_MERCHANT_MODULE", "wc_redsys_" . $this->version );
    	$request->useReference($ref[0]);
    	$ip = $_SERVER['REMOTE_ADDR'] == "::1" ? "127.0.0.1" : $_SERVER['REMOTE_ADDR'];
		$request->addParameter ( "DS_MERCHANT_CLIENTIP", $ip );
// 		if($this->withdcc)
// 			$request->withDcc();
    
    	$service = new ISService ( $this->clave256, $this->entorno );
    	$result = $service->sendOperation ( $request );
    
    	if ($result->getResult () == ISConstants::$RESP_LITERAL_OK) {
    		$order->update_status($this->estado,__( 'Order accepted: payment registered successfully', 'woocommerce' ));
    			
    		$reference=$result->getOperation()->getMerchantIdentifier();
    		if($reference!=null){
    			$idCustomer=get_post_meta($origIdCart, '_customer_user', true);
    			$cardNumber=$result->getOperation()->getCardNumber();
    			$brand=$result->getOperation()->getCardBrand();
    			$cardType=$result->getOperation()->getCardType();
    
    			WC_Redsys_Ref::saveReference($idCustomer, $reference, $cardNumber, $brand, $cardType);
    		}
    
    		header('Content-Type: application/json');
    		die(json_encode(array("redir"=>true, "url"=>$this->get_return_url($order))));
    	}
    	else{
    		$order->update_status('cancelled',__( 'Order cancelled: denied operation', 'woocommerce' ));
    
    		header('Content-Type: application/json');
    		die(json_encode(array("redir"=>true, "url"=>str_replace("&amp;","&",$order->get_cancel_order_url()))));
    	}
    }
    
    function redirect_to_tdsecure(){
    	$form='<iframe name="redsys_iframe_acs" name="redsys_iframe_acs" src=""
	    		id="redsys_iframe_acs"
	    		sandbox="allow-same-origin allow-scripts allow-top-navigation allow-forms"
	    		height="95%" width="100%" style="border: none; display: none;"></iframe>
	    	
    	<form name="redsysAcsForm" id="redsysAcsForm"
    		action="'.WC()->session->get( "REDSYS_urlacs" ).'" method="POST"
    		target="redsys_iframe_acs" style="border: none;">
    		<table name="dataTable" border="0" cellpadding="0">
    			<input type="hidden" name="PaReq"
    				value="'.WC()->session->get( "REDSYS_pareq" ).'">
    			<input type="hidden" name="TermUrl"
    				value="'.$this->secure_back_url.'">
    			<input type="hidden" name="MD"
    				value="'.WC()->session->get( "REDSYS_md" ).'">
    			<br>
    			<p
    				style="font-family: Arial; font-size: 16; font-weight: bold; color: black; align: center;">
    				Conectando con el emisor...</p>
    		</table>
    	</form>
	    	
    	<script>
    		window.onload = function () {
    		    document.getElementById("redsys_iframe_acs").onload = function() {
    		    	document.getElementById("redsysAcsForm").style.display="none";
    		    	document.getElementById("redsys_iframe_acs").style.display="inline";
    		    }
    			document.redsysAcsForm.submit();
    		}
    	</script>';

    	WC()->session->set('REDSYS_pareq', null);
    	WC()->session->set('REDSYS_urlacs', null);
    	WC()->session->set('REDSYS_md', null);
    	
    	die($form);
    }
    
    function back_from_tdsecure(){
    	$operation = unserialize(WC()->session->get( "REDSYS_oper" ));
    	WC()->session->set('REDSYS_oper', null);
    		
    	$request = new ISAuthenticationMessage ();
    	$request->setOrder ( $operation->getOrder () );
    	$request->setAmount ( $operation->getAmount () );
    	$request->setCurrency ( $operation->getCurrency () );
    	$request->setMerchant ( $operation->getMerchant () );
    	$request->setTerminal ( $operation->getTerminal () );
    	$request->setTransactionType ( $operation->getTransactionType () );
		$request->addEmvParameter ( ISConstants::$RESPONSE_JSON_THREEDSINFO_ENTRY , ISConstants::$RESPONSE_3DS_CHALLENGE_RESPONSE );
		$request->addEmvParameter ( ISConstants::$RESPONSE_JSON_PROTOCOL_VERSION_ENTRY , ISConstants::$RESPONSE_3DS_VERSION_1 );
		$request->addEmvParameter ( ISConstants::$RESPONSE_JSON_PARES_ENTRY , $_POST ["PaRes"] );
		$request->addEmvParameter ( ISConstants::$RESPONSE_JSON_MD_ENTRY , $_POST ["MD"] );
    		
    	$service = new ISAuthenticationService ( $this->clave256, $this->entorno );
    	$result = $service->sendOperation ( $request );
    		
    	$origIdCart=WC()->session->get("REDSYS_idCart");
    	WC()->session->set('REDSYS_idCart', null);
    	
    	$order=new WC_Order($origIdCart);
    		
    	if ($result->getResult () == ISConstants::$RESP_LITERAL_OK) {
    		$order->update_status($this->estado,__( 'Order accepted: payment registered successfully', 'woocommerce' ));
			$urlDst=$this->get_return_url($order);
			
			$reference=$result->getOperation()->getMerchantIdentifier();
			if($reference!=null){
				$idCustomer=get_post_meta($origIdCart, '_customer_user', true);
				$cardNumber=$result->getOperation()->getCardNumber();
				$brand=$result->getOperation()->getCardBrand();
				$cardType=$result->getOperation()->getCardType();
				
				WC_Redsys_Ref::saveReference($idCustomer, $reference, $cardNumber, $brand, $cardType);
			}
    	} else {
			$order->update_status('cancelled',__( 'Order cancelled: denied operation', 'woocommerce' ));
			$urlDst=str_replace("&amp;","&",$order->get_cancel_order_url());
    	}
    	
    	$form= '<p style="font-family: Arial; font-size: 16; font-weight: bold; color: black; align: center;">
					Procesando operación...
				</p>
				<script>
					window.top.top.location.href="'.$urlDst.'";
				</script>';
    	
    	die($form);
    }

    function process_payment( $order_id ) {
        global $woocommerce;
        $order = new WC_Order($order_id);

        // Return receipt_page redirect
        return array(
            'result' 	=> 'success',
            'redirect'	=> $order->get_checkout_payment_url( true )
        );
    }
    
    function receipt_page( $order ) {
		ISLogger::generateIdLog(true);
    	$allowReference=$this->withref=="1";
    	$body_style=$this->body_style;
    	$form_style=$this->form_style;
    	$form_text_style=$this->form_text_style;
    	$btnStyle=$this->button_style;
    	$btnText=$this->button_text;

    	//Objeto tipo pedido
    	$orderWC = new WC_Order($order);
    	
    	$html=file_get_contents(ABSPATH.'/wp-content/plugins/redsys/pages/templates/paymentform.html');
    	$redsysJs=ISConstants::getJSPath($this->entorno);
    	$staticPath=home_url().'/wp-content/plugins/redsys/pages/assets';    
    	$urlKo=$orderWC->get_cancel_order_url();
		$orderId=str_pad ( $order, 12, "0", STR_PAD_LEFT );
		$isLogged = is_user_logged_in();
		$userId=get_post_meta($order, '_customer_user', true);
		ISLogger::debug("ID del usuario cargado: ".$userId);

		if ($isLogged == true) {
			ISLogger::debug("El usuario que hace el pedido SI está logeado");
		}
		if ($isLogged == false) {
			ISLogger::debug("El usuario que hace el pedido NO está logeado");
		}

		$brandImg="";
		$refTitle="";
		if($allowReference && $isLogged){
			$ref=WC_Redsys_Ref::getCustomerRef($userId);
			if($ref!=null && $ref[2]!=null)
				$brandImg='<img src="'.home_url().'/wp-content/plugins/redsys/pages/assets/images/brands/'.$ref[2].'.jpg" style="display: inline;"/>';
			
			if($ref!=null){
				$refTitle="Usar tarjeta guardada ";
				if($ref[3]=="C")
					$refTitle="Usar tarjeta de crédito guardada ";
				else
					$refTitle="Usar tarjeta de débito guardada ";
				
				if($ref[1]!=null)
					$refTitle.=$ref[1];
			}
		}

		$html=str_replace("{merchantCode}",$this->fuc,$html);
		$html=str_replace("{merchantTerminal}",$this->terminal,$html);
    	$html=str_replace("{redsysJs}",$redsysJs,$html);
    	$html=str_replace("{staticPath}",$staticPath,$html); 
    	$html=str_replace("{origIdCart}",$order,$html);    
    	$html=str_replace("{idCart}",$orderId,$html);    
    	$html=str_replace("{allowReference}",$allowReference?"true":"false",$html);
    	$html=str_replace("{displayReference}",!$allowReference || !$isLogged?'display: none; ':'',$html);
    	$html=str_replace("{displayRadios}",$ref!=null?'':'display: none; ',$html);
    	$html=str_replace("{referenceTitle}",$refTitle,$html);
    	$html=str_replace("{cardBrandLogo}",$brandImg,$html);
    	$html=str_replace("{procUrl}",$this->process_url,$html);
    	$html=str_replace("{procUrlRef}",$this->ref_process_url,$html);
    	$html=str_replace("{body_style}",$body_style,$html);
    	$html=str_replace("{form_style}",$form_style,$html);
    	$html=str_replace("{form_text_style}",$form_text_style,$html);
    	$html=str_replace("{btnStyle}",$btnStyle,$html);
    	$html=str_replace("{btnText}",$btnText,$html);
    	$html=str_replace("{urlKo}",str_replace("&amp;","&",$urlKo),$html);
    	
    	
    	echo $html;
    }

    function init_form_fields() {
    	global $woocommerce;
    
    	$this->form_fields = array(
    			'enabled' => array(
    					'title'       => __( 'Activar Redsys:', 'woocommerce' ),
    					'label'       => __( 'Activar pago Redsys', 'woocommerce' ),
    					'type'        => 'checkbox',
    					'description' => '',
    					'default'     => 'yes'
    			),
    			'title' => array(
    					'title'       => __( 'Título', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'Payment method description that the customer will see on your checkout.', 'woocommerce' ),
    					'default'     => __( 'Pago con tarjeta', 'woocommerce' ),
    					'desc_tip'    => true,
    			),
    			'description' => array(
    					'title'       => __( 'Descripción', 'woocommerce' ),
    					'type'        => 'textarea',
    					'description' => __( 'Payment method description that the customer will see on your website.', 'woocommerce' ),
    					'default'     => __( 'Esta es la opción de la pasarela de pago con tarjeta integrada en el comercio. Los datos de tarjeta son almacenados de manera segura por la entidad.', 'woocommerce' ),
    					'desc_tip'    => true,
    			),
    			'entorno' => array(
    					'title'       => __( 'Entorno de Redsys', 'woocommerce' ),
    					'type'        => 'select',
    					'description' => __( 'Entorno del proceso de pago.', 'woocommerce' ),
    					'default'     => '0',
    					'desc_tip'    => true,
    					'options'     => array(
    							'0' => __( 'Pruebas', 'woocommerce' ),
    							'1' => __( 'Real', 'woocommerce' )
    					)
    			),
    			'fuc' => array(
    					'title'       => __( 'FUC Comercio', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'FUC del comercio.', 'woocommerce' ),
    					'default'     => __( '', 'woocommerce' ),
    					'desc_tip'    => true
    			),
    			'terminal' => array(
    					'title'       => __( 'Terminal', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'Terminal del comercio.', 'woocommerce' ),
    					'default'     => __( '1', 'woocommerce' ),
    					'desc_tip'    => true,
    			),
    			'clave256' => array(
    					'title'       => __( 'Clave secreta de encriptación (SHA-256)', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'Clave del comercio.', 'woocommerce' ),
    					'default'     => __( '', 'woocommerce' ),
    					'desc_tip'    => true,
    			),
    			'moneda' => array(
    					'title'       => __( 'Tipo de Moneda', 'woocommerce' ),
    					'type'        => 'select',
    					'description' => __( 'Moneda del proceso de pago.', 'woocommerce' ),
    					'default'     => '978',
    					'desc_tip'    => true,
    					'options'     => array(
    						'978' => __( 'EURO', 'woocommerce' ),
    						'840' => __( 'DOLAR', 'woocommerce' ),
    						'826' => __( 'LIBRA', 'woocommerce' )
    					)
    			),
    			'trans' => array(
    					'title'       => __( 'Tipo de Transacción', 'woocommerce' ),
    					'type'        => 'select',
    					'description' => __( 'Tipo de Transacción del comercio.', 'woocommerce' ),
    					'default'     => '0',
    					'desc_tip'    => true,
    					'options'     => array(
    						'0' => __( 'Autorización', 'woocommerce' ),
    						'1' => __( 'Preautorización', 'woocommerce' )
    					)
    			),
    			'activar_log' => array(
    					'title'       => __( 'Activar Log', 'woocommerce' ),
    					'type'        => 'select',
    					'description' => __( 'Activar trazas de log.', 'woocommerce' ),
    					'default'     => '0',
    					'desc_tip'    => true,
    					'options'     => array(
    							'0' => __( 'No', 'woocommerce' ),
    							'1' => __( 'Info', 'woocommerce' ),
    							'3' => __( 'Debug', 'woocommerce' )
    					)
    			),
    			'estado' => array(
    					'title'       => __( 'Estado', 'redsys_wc' ),
    					'type'        => 'select',
    					'description' => __( 'Estado tras el pago.', 'redsys_wc' ),
    					'default'     => 'no',
    					'desc_tip'    => true,
    					'options'     => array()
    			),
//     			'withdcc' => array(
//     					'title'       => __( 'Habilitar DCC', 'woocommerce' ),
//     					'type'        => 'select',
//     					'description' => __( 'Habilita el servicio DCC.', 'woocommerce' ),
//     					'default'     => '0',
//     					'desc_tip'    => true,
//     					'options'     => array(
//     						'0' => __( 'No', 'woocommerce' ),
//     						'1' => __( 'Si', 'woocommerce' )
//     					)
//     			),
    			'with3ds' => array(
    					'title'       => __( 'Habilitar pago seguro', 'woocommerce' ),
    					'type'        => 'select',
    					'description' => __( 'Habilita el servicio de autenticación de titular.', 'woocommerce' ),
    					'default'     => '0',
    					'desc_tip'    => true,
    					'options'     => array(
    						'0' => __( 'No', 'woocommerce' ),
    						'1' => __( 'Si', 'woocommerce' )
    					)
    			),
    			'withref' => array(
    					'title'       => __( 'Habilitar pago por referencia', 'woocommerce' ),
    					'type'        => 'select',
    					'description' => __( 'Habilita el pago por referencia .', 'woocommerce' ),
    					'default'     => '0',
    					'desc_tip'    => true,
    					'options'     => array(
    						'0' => __( 'No', 'woocommerce' ),
    						'1' => __( 'Si', 'woocommerce' )
    					)
    			),
    			'button_text' => array(
    					'title'       => __( 'Texto del botón de pago', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'Texto del botón de pago.', 'woocommerce' ),
    					'default'     => 'REALIZAR PAGO',
    					'desc_tip'    => true
    			),
    			'button_style' => array(
    					'title'       => __( 'Estilo del botón de pago', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'Estilo del botón de pago.', 'woocommerce' ),
    					'default'     => 'background-color:orange;color:black;',
    					'desc_tip'    => true
    			),
    			'body_style' => array(
    					'title'       => __( 'Estilo del iframe', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'Estilo del iframe.', 'woocommerce' ),
    					'default'     => 'color:black;',
    					'desc_tip'    => true
    			),
    			'form_style' => array(
    					'title'       => __( 'Estilo del formulario', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'Estilo del formulario.', 'woocommerce' ),
    					'default'     => 'color:grey;',
    					'desc_tip'    => true
    			),
    			'form_text_style' => array(
    					'title'       => __( 'Estilo del texto de formulario', 'woocommerce' ),
    					'type'        => 'text',
    					'description' => __( 'Estilo del texto de formulario.', 'woocommerce' ),
    					'default'     => ';',
    					'desc_tip'    => true
    			)
    	);
    		
    	$tmp_estados=wc_get_order_statuses();
    	foreach($tmp_estados as $est_id=>$est_na){
    		$this->form_fields['estado']['options'][substr($est_id,3)]=$est_na;
    	}
    }
}